#include "fintek_api.h"

void usage(int argc, char *argv[])
{
	fprintf(stderr, "\n");
	fprintf(stderr, "EEPROM Read/Write Demo: \n");
	fprintf(stderr, "%s <r,w,d> <reg> <val>\n\n", argv[0]);
	fprintf(stderr, "Examples:\n");
	fprintf(stderr, "\tRead reg 0x30: \n\t\t%s r 0x30 \n", argv[0]);
	fprintf(stderr, "\tWrite reg 0x30, val 0x1f:\n\t\t%s w 0x30 0x1f\n",
		argv[0]);
	fprintf(stderr, "\tDump all reg:\n\t\t%s d\n", argv[0]);
}

int eeprom_dump(void)
{
	int i;
	int status;
	unsigned int data;

	for (i = 0x00; i <= 0xff; ++i) {
		status = GetEEPROMValue(i, &data);
		if (status) {
			fprintf(stderr, "%s: Read EEPROM Error\n", __func__);
			return -1;
		}

		if (!(i % 16))
			printf("\n");

		printf("%02x ", data);
	}

	printf("\n");

	return 0;
}

int eeprom_demo(int argc, char *argv[])
{
	int i;
	int status;
	unsigned int reg;
	unsigned int data;
	FILE *file;

	status = InitEEPROM();
	if (status) {
		fprintf(stderr, "%s: InitEEPROM Error\n", __func__);
		return status;
	}

	do {
		switch (argc) {
		case 2:
			if (argv[1][0] == 'd')
				return eeprom_dump();
		case 3:
			if (argv[1][0] == 'r') {
				if (argv[2] == NULL)
					break;

				reg = strtol(argv[2], NULL, 16) & 0xff;

				status = GetEEPROMValue(reg, &data);
				if (status) {
					fprintf(stderr, "%s: read failed\n",
						__func__);
					return status;
				}

				printf("Read reg:0x%02x, value:0x%02x\n",
				       reg & 0xff, data & 0xff);
				return 0;
			}

		case 4:
			if (argv[1][0] == 'w') {
				if ((argv[2] == NULL) || (argv[3] == NULL))
					break;

				reg = strtol(argv[2], NULL, 16) & 0xff;
				data = strtol(argv[3], NULL, 16) & 0xff;

				status = SetEEPROMValue(reg, data);
				if (status) {
					fprintf(stderr, "%s: read failed\n",
						__func__);
					return status;
				}

				printf("Write reg:0x%02x, value:0x%02x\n",
				       reg & 0xff, data & 0xff);
				return 0;
			}
		}

		usage(argc, argv);
		return -1;
	} while (0);

	return 0;
}

int main(int argc, char *argv[])
{
	int nRet = 0;
	sFintek_sio_data sio_data;

	/* Please set this to 1 enable more debug message */
	set_debug(0);

	fprintf(stderr, "FINTEK Lib Version: %s\n", getFintekLibVersion());

	if (nRet = init_fintek_sio(eSIO_TYPE_F81504, 0, &sio_data)) {
		fprintf(stderr, "init_fintek_sio error\n");
		exit(3);
	}

	ActiveSIO(sio_data.ic_port, sio_data.key);
	nRet = eeprom_demo(argc, argv);
	DeactiveSIO(sio_data.ic_port);

	return nRet;
}
