#include "fintek_api.h"

void usage(int argc, char *argv[])
{
	fprintf(stderr, "\n\n\n\nFintek GPIO Demo API:\n");
	fprintf(stderr, "%s <idx> <direct> <type> <value> <pull_mode>\n\n", argv[0]);
	fprintf(stderr, "Param List:\n");
	fprintf(stderr, "\tidx: number of GPIO, lke GPIO17, idx=0x17\n");
	fprintf(stderr, "\tdir: 0: In, 1: Out\n");
	fprintf(stderr, "\ttype: 0: Open Drain, 1: Push-Pull\n");
	fprintf(stderr, "\tvalue: 0/1 when single mode, bitwise when group mode\n");
	fprintf(stderr, "\tpull_mode (option): 0: Pull_Low, 0: Pull_High, 0: Pull_Disable,\n");
}

int input_verify(int idx)
{
	unsigned int data = 0;
	int status;
	eGPIO_Pull_Mode pull_mode;
	eGPIO_Drive_Mode drive_mode;
	eGPIO_Direction dir_mode;
	char *name;

	CHECK_RET(_SetGpioOutputEnableIdx(idx, eGPIO_Direction_In));

	status = _GetGpioOutputEnableIdx(idx, &dir_mode);
	switch (dir_mode) {
	case eGPIO_Direction_Out:
		name = "eGPIO_Direction_Out";
		break;
	case eGPIO_Direction_In:
		name = "eGPIO_Direction_In";
		break;
	case eGPIO_Direction_Invalid:
		name = "eGPIO_Direction_Invalid";
		break;
	}
	printf("%s: status: %d, dir_mode: %s\n", __func__, status, name);

	status = _GetGpioInputDataIdx(idx, &data);
	printf("%s: status: %d, input_data: %d\n", __func__, status, data);
}

int output_verify(int idx)
{
	unsigned int data = 0;
	int status;
	eGPIO_Pull_Mode pull_mode;
	eGPIO_Drive_Mode drive_mode;
	eGPIO_Direction dir_mode;
	char *name;

	CHECK_RET(_SetGpioOutputEnableIdx(idx, eGPIO_Direction_Out));

	CHECK_RET(_SetGpioDriveEnable(idx, eGPIO_Drive_Mode_OpenDrain));
	CHECK_RET(_SetGpioPullMode(idx, eGPIO_Pull_Disable));
	CHECK_RET(_SetGpioOutputDataIdx(idx, 1));

	status = _GetGpioPullMode(idx, &pull_mode);
	switch (pull_mode) {
	case eGPIO_Pull_Low:
		name = "eGPIO_Pull_Low";
		break;
	case eGPIO_Pull_High:
		name = "eGPIO_Pull_High";
		break;
	case eGPIO_Pull_Disable:
		name = "eGPIO_Pull_Disable";
		break;
	case eGPIO_Pull_Invalid:
		name = "eGPIO_Pull_Invalid";
		break;
	}
	printf("%s: status: %d, pull_mode: %s\n", __func__, status, name);

	status = _GetGpioDriveEnable(idx, &drive_mode);
	switch (drive_mode) {
	case eGPIO_Drive_Mode_OpenDrain:
		name = "eGPIO_Drive_Mode_OpenDrain";
		break;
	case eGPIO_Drive_Mode_Pushpull:
		name = "eGPIO_Drive_Mode_Pushpull";
		break;
	case eGPIO_Drive_Mode_Invalid:
		name = "eGPIO_Drive_Mode_Invalid";
		break;
	}
	printf("%s: status: %d, drive_mode: %s\n", __func__, status, name);

	status = _GetGpioOutputEnableIdx(idx, &dir_mode);
	switch (dir_mode) {
	case eGPIO_Direction_Out:
		name = "eGPIO_Direction_Out";
		break;
	case eGPIO_Direction_In:
		name = "eGPIO_Direction_In";
		break;
	case eGPIO_Direction_Invalid:
		name = "eGPIO_Direction_Invalid";
		break;
	}
	printf("%s: status: %d, dir_mode: %s\n", __func__, status, name);

	status = _GetGpioOutputDataIdx(idx, &data);
	printf("%s: status: %d, output_data: %d\n", __func__, status, data);
}

int gpio_demo(int argc, char *argv[])
{
	unsigned int idx = 0;
	unsigned int data = 0;
	unsigned int pull_mode = -1;
	eGPIO_Direction dir_mode;
	char *desc;
	int status;

	if (argc < 5) {
		usage(argc, argv);
		exit(3);
	}

	idx = strtol(argv[1], NULL, 16);

	if (argc >= 6) {
		pull_mode = strtol(argv[5], NULL, 10);
		if (pull_mode >= eGPIO_Pull_Invalid) {
			printf("pull_mode >= eGPIO_Pull_Invalid failed\n");
			return 0;
		}
	}
#if 0
	output_verify(idx);
	//input_verify(idx);

#else
	CHECK_RET(_EnableGPIO(idx, eGPIO_Mode_Enable));
	printf("idx: 0x%x => set:%x, count:%x\n", idx, (idx >> 4) & 0xf, (idx >> 0) & 0xf);

	// out
	data = strtol(argv[2], NULL, 16);
	CHECK_RET(_SetGpioOutputEnableIdx(idx, data));
	printf("Dir: 0x%x => %s\n", data, (data == eGPIO_Direction_Out) ? "OUT" : "IN");

	if (pull_mode != -1 && (pull_mode < eGPIO_Pull_Invalid)) {
		CHECK_RET(_SetGpioPullMode(idx, pull_mode));
		switch (pull_mode) {
		case eGPIO_Pull_Low:
			desc = "eGPIO_Pull_Low";
			break;
		case eGPIO_Pull_High:
			desc = "eGPIO_Pull_High";
			break;
		case eGPIO_Pull_Disable:
			desc = "eGPIO_Pull_Disable";
			break;
		}

		printf("Pull Mode: %s\n", desc);
	}

	if (data == eGPIO_Direction_Out) {

		data = strtol(argv[3], NULL, 16);
		CHECK_RET(_SetGpioDriveEnable(idx, data));
		printf("Driving Mode: 0x%x => %s\n", data, (data == eGPIO_Drive_Mode_OpenDrain)
		       ? "OpenDrain" : "Push-Pull");

		data = strtol(argv[4], NULL, 16);

		/* Single mode */
		status = _SetGpioOutputDataIdx(idx, data);
		if (!status) {
			printf("Write Value: 0x%x \n", data);
		} else {
			/* Group mode */
			status = _SetGpioGroupOutputDataIdx(idx >> 4, data);
			if (!status) {
				printf("Write Group Value: 0x%x \n", data);
			} else {
				printf("Write single/group failed\n");
			}
		}

	} else {
		status = _GetGpioInputDataIdx(idx, &data);
		if (status) {
			printf("This IC not support Single Input mode\n");

			status = _GetGpioGroupInputDataIdx(idx >> 4, &data);
			if (!status) {
				printf("Set: %x data: %x, Pin: %x: data: %x\n",
				       idx >> 4, data, idx, (int)(data & BIT(idx & 0xf)));

				return 0;
			} else {
				printf("This IC not support Group Input mode\n");
			}
		} else {
			printf("Read Pin: %x Value: 0x%x \n", idx, data);

		}
	}
#endif
	return 0;
}

int main(int argc, char *argv[])
{
	int nRet = 0;
	sFintek_sio_data sio_data;

	/* Please set this to 1 enable more debug message */
	set_debug(0);

	fprintf(stderr, "FINTEK Lib Version: %s\n", getFintekLibVersion());

	if (nRet = init_fintek_sio(eSIO_TYPE_F81962, 0, &sio_data)) {
		fprintf(stderr, "init_fintek_sio error\n");
		exit(3);
	}

	ActiveSIO(sio_data.ic_port, sio_data.key);
	gpio_demo(argc, argv);
	DeactiveSIO(sio_data.ic_port);

	return 0;
}
